import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import javax.sound.midi.*;
import java.beans.*;

/*
 * The GUI-class representing the popupdialogs in the Track menu.
 *
 * @author Bjrn Lanneskog
 */
 
public class MooDialog extends JDialog {

	private JLabel labelA, labelB, labelC;
	private JTextField textFieldA, textFieldB, textFieldC;
	private JComboBox trackListA, trackListB; 
	private JButton okButton, cancelButton;
	public static final int ADD_TRACK = 1,
				DELETE_TRACK = 2,
				COPY_TRACK = 3,
				MOVE_TRACK = 4,
				JUMP = 5;
	
	/*
	 * Constructor of the dialogs.
	 */
	 public MooDialog(int type) {
	 	super(Moosique.getGUI(), false);
			
		Container pane = getContentPane();
		pane.setLayout(null);
		
		Track[] tracks = Moosique.getSequence().getTracks();

		switch (type) {
			case ADD_TRACK:
				
				makeAddDialog(pane, tracks);
				
				break;
				
			case DELETE_TRACK:
			
				makeDelDialog(pane, tracks);
				
				break;
				
			case COPY_TRACK:
				
				makeCopyDialog(pane, tracks);
			
				break;
			
			case MOVE_TRACK:
				
				makeMoveDialog(pane, tracks);
				
				break;
			
			case JUMP:
				
				makeJumpDialog(pane);
				
				break;
		}
	 }
	
	/*
	 * Builds the add track popupdialog
	 */
	private void makeAddDialog(Container pane, Track[] tracks) {
	
		setTitle("Add track");
		labelA = new JLabel("Name of track", JLabel.CENTER);
		pane.add(labelA);
		textFieldA = new JTextField();	
		pane.add(textFieldA);
		labelB = new JLabel("Add after", JLabel.CENTER);
		pane.add(labelB);
		trackListA = new JComboBox();
		for (int i = 1; i <= tracks.length; i++) trackListA.addItem("Track " + i);
		pane.add(trackListA);
		cancelButton = new JButton("Cancel");
		pane.add(cancelButton);
		okButton = new JButton("OK");
		pane.add(okButton);
	
		labelA.setBounds(50, 10, 100, 20);
		textFieldA.setBounds(40, 35, 120, 20);
		labelB.setBounds(50, 70, 100, 20);
		trackListA.setBounds(40, 95, 120, 20);
		cancelButton.setBounds(10, 150, 80, 30);
		okButton.setBounds(120, 150, 60, 30);
	
		setLocation((Toolkit.getDefaultToolkit().getScreenSize().width - this.getWidth()) / 2,
		(Toolkit.getDefaultToolkit().getScreenSize().height - this.getHeight()) / 2);
		setResizable(false);
		pack();
		setSize(200,220);
		setVisible(true);
	}
	
	/*
	 * Builds the delete track popupdialog
	 */
	private void makeDelDialog(Container pane, Track[] tracks) {
	
		setTitle("Delete track");
		labelB = new JLabel("Delete track", JLabel.CENTER);
		pane.add(labelB);
		trackListA = new JComboBox();
		for (int i = 1; i <= tracks.length; i++) trackListA.addItem("Track " + i);
		pane.add(trackListA);
		cancelButton = new JButton("Cancel");
		pane.add(cancelButton);
		okButton = new JButton("OK");
   		pane.add(okButton);
			
		labelB.setBounds(50, 10, 100, 20);
		trackListA.setBounds(40, 35, 120, 20);
		cancelButton.setBounds(10, 90, 80, 30);
		okButton.setBounds(120, 90, 60, 30);
	
		setLocation((Toolkit.getDefaultToolkit().getScreenSize().width - this.getWidth()) / 2,
		(Toolkit.getDefaultToolkit().getScreenSize().height - this.getHeight()) / 2);
		setResizable(false);
		pack();
		setSize(200,165);
		setVisible(true);
	}
	
	/*
	 * Builds the copy track popupdialog
	 */
	private void makeCopyDialog(Container pane, Track[] tracks) {
	
		setTitle("Copy Track");
		labelA = new JLabel("Track to copy", JLabel.CENTER);
		pane.add(labelA);
		trackListA = new JComboBox();
		for (int i = 1; i <= tracks.length; i++) trackListA.addItem("Track " + i);
		pane.add(trackListA);
		labelB = new JLabel("Insert after", JLabel.CENTER);
		pane.add(labelB);
		trackListB = new JComboBox();
		for (int i = 1; i <= tracks.length; i++) trackListB.addItem("Track " + i);
		pane.add(trackListB);
		cancelButton = new JButton("Cancel");
		pane.add(cancelButton);
		okButton = new JButton("OK");
    		pane.add(okButton);
		
		labelA.setBounds(50, 10, 100, 20);
		trackListA.setBounds(40, 35, 120, 20);
		labelB.setBounds(50, 70, 100, 20);
		trackListB.setBounds(40, 95, 120, 20);
		cancelButton.setBounds(10, 150, 80, 30);
		okButton.setBounds(120, 150, 60, 30);
	
		setLocation((Toolkit.getDefaultToolkit().getScreenSize().width - this.getWidth()) / 2,
		(Toolkit.getDefaultToolkit().getScreenSize().height - this.getHeight()) / 2);
		setResizable(false);
		pack();
		setSize(200,220);
		setVisible(true);
	}
	
	/*
	 * Builds the move track popupdialog
	 */
	private void makeMoveDialog(Container pane, Track[] tracks) {
	
		setTitle("Move track");
		labelA = new JLabel("Track to move", JLabel.CENTER);
		pane.add(labelA);
		trackListA = new JComboBox();
		for (int i = 1; i <= tracks.length; i++) trackListA.addItem("Track " + i);
		pane.add(trackListA);
		labelB = new JLabel("Insert after", JLabel.CENTER);
		pane.add(labelB);
		trackListB = new JComboBox();
		for (int i = 1; i <= tracks.length; i++) trackListB.addItem("Track " + i);
		pane.add(trackListB);
		cancelButton = new JButton("Cancel");
		pane.add(cancelButton);
		okButton = new JButton("OK");
      		pane.add(okButton);
		
		labelA.setBounds(40, 10, 120, 20);
		trackListA.setBounds(40, 35, 120, 20);
		labelB.setBounds(50, 70, 100, 20);
		trackListB.setBounds(40, 95, 120, 20);
		cancelButton.setBounds(10, 150, 80, 30);
		okButton.setBounds(120, 150, 60, 30);
	
		setLocation((Toolkit.getDefaultToolkit().getScreenSize().width - this.getWidth()) / 2,
		(Toolkit.getDefaultToolkit().getScreenSize().height - this.getHeight()) / 2);
		setResizable(false);
		pack();
		setSize(200,220);
		setVisible(true);
	}
	
	/*
	 * Builds the jump popupdialog
	 */
	private void makeJumpDialog(Container pane) {
		
		setTitle("Jump");
		labelA = new JLabel("Msr", JLabel.CENTER);
		pane.add(labelA);
		labelB = new JLabel("Beat", JLabel.CENTER);
		pane.add(labelB);
		labelC = new JLabel("Tick", JLabel.CENTER);
		pane.add(labelC);
		textFieldA = new JTextField();
		pane.add(textFieldA);
		textFieldB = new JTextField();
		pane.add(textFieldB);
		textFieldC = new JTextField();
		pane.add(textFieldC);
		cancelButton = new JButton("Cancel");
		pane.add(cancelButton);
		okButton = new JButton("OK");
      		pane.add(okButton);
      		
      		labelA.setBounds(30, 25, 50, 20);
      		labelB.setBounds(105, 25, 50, 20);
      		labelC.setBounds(180, 25, 50, 20);
      		textFieldA.setBounds(30, 45, 50, 20);
      		textFieldB.setBounds(105, 45, 50, 20);
      		textFieldC.setBounds(180, 45, 50, 20);
      		cancelButton.setBounds(35, 90, 80, 30);
		okButton.setBounds(155, 90, 60, 30);
		
		setLocation((Toolkit.getDefaultToolkit().getScreenSize().width - this.getWidth()) / 2,
		(Toolkit.getDefaultToolkit().getScreenSize().height - this.getHeight()) / 2);
		setResizable(false);
		pack();
		setSize(260,175);
		setVisible(true);
	}
	
	private MooNote note;
	private JOptionPane optionPane;
	private JTextField pitch;
	private JTextField velocity;
	private JTextField length;
	
	/** 
	 * Creates a new note preference dialog.
	 * @param mn	the note that will be graphically represented
	 */
	public MooDialog(MooNote mn) {
		super(Moosique.getGUI(), "Note properties", false);
		note = mn;
		pitch = new JTextField(new Integer(note.getPitch()).toString(),3);
		JPanel pitchpanel = new JPanel();
		pitchpanel.add(new Label("Pitch: "));
		pitchpanel.add(pitch);


		velocity = new JTextField(new Integer(note.getVelocity()).toString(),3);
		JPanel velocitypanel = new JPanel();
		velocitypanel.add(new Label("Velocity: "));
		velocitypanel.add(velocity);

		length = new JTextField(new Integer(note.getDuration()).toString(),5);
		JPanel lengthpanel = new JPanel();
		lengthpanel.add(new Label("Length: "));
		lengthpanel.add(length);

	        Object[] array = {"Set the note properties",
				pitchpanel,
				velocitypanel,
				lengthpanel};
	
	        final String btnString1 = "Enter";
	        final String btnString2 = "Cancel";
	        Object[] options = {btnString1, btnString2};
	
	        optionPane = new JOptionPane(array, 
	                                    JOptionPane.QUESTION_MESSAGE,
	                                    JOptionPane.YES_NO_OPTION,
	                                    null,
	                                    options,
	                                    options[0]);
	        setContentPane(optionPane);
	        setDefaultCloseOperation(DISPOSE_ON_CLOSE);
	
	        ActionListener intValidator = new ActionListener() {
	            public void actionPerformed(ActionEvent e) {
					if (e.getSource() instanceof JTextField){
						JTextField s = (JTextField)e.getSource();
						int num = Integer.parseInt(s.getText());
						if (num < 0)
							num = 0;
						else if (num > 127 && s != length)
							num = 127;
						s.setText(new Integer(num).toString());
					}
	            }
	        };
		pitch.addActionListener(intValidator);
		velocity.addActionListener(intValidator);
		length.addActionListener(intValidator);

	        optionPane.addPropertyChangeListener(new PropertyChangeListener() {
	            public void propertyChange(PropertyChangeEvent e) {
	                String prop = e.getPropertyName();
	
	                if (isVisible() 
	                 && (e.getSource() == optionPane)
	                 && (prop.equals(JOptionPane.VALUE_PROPERTY) ||
	                     prop.equals(JOptionPane.INPUT_VALUE_PROPERTY))) {
	                    Object value = optionPane.getValue();
	
	                    if (value == JOptionPane.UNINITIALIZED_VALUE) {
	                        //ignore reset
	                        return;
	                    }
	
	                    // Reset the JOptionPane's value.
	                    // If you don't do this, then if the user
	                    // presses the same button next time, no
	                    // property change event will be fired.
	                    optionPane.setValue(
	                            JOptionPane.UNINITIALIZED_VALUE);
	
	                    if (value.equals(btnString1)) {
							note.setPitch(Integer.parseInt(pitch.getText()));
							note.setVelocity(Integer.parseInt(velocity.getText()));
							note.setDuration(Integer.parseInt(length.getText()));
	
	                        setVisible(false);
	                    } else { // user closed dialog or clicked cancel
	                        setVisible(false);
	                    }
	                }
	            }
	        });
		pack();
	}
}
